pub struct SignatureGated<T> { /* private fields */ }
A SignatureGated object is a self-signed object that’s well-signed when one or more ValidatableSignature objects are correct.

Implementations

Return a new SignatureGated object that will be treated as correct if every one if the given set of signatures is valid.