pub struct TimeRangeBoundBuilder(/* private fields */);Expand description
Accumulator used by TimeRangeBounds::build_intersect
Provided to the user’s logic callback by TimeRangeBound::build_intersect
There is no other way to obtain a TimeRangeBoundBuilder.
Implementations
Sourceimpl TimeRangeBoundBuilder
impl TimeRangeBoundBuilder
Sourcepub fn incorporate_unwrap<Component: TimeBound>(
&mut self,
component: Component,
) -> Component::Inner
pub fn incorporate_unwrap<Component: TimeBound>( &mut self, component: Component, ) -> Component::Inner
Handle a TimeBound, ensuring its validity range will be honoured
This is equivalent to TimeBound::unwrap_with,
which is normally more convenient.
CORRECTNESS
See TimeBound::unwrap_with and TimeRangeBound::build_intersect.
Sourcepub fn intersect_bounds(&mut self, bounds: TimeRange)
pub fn intersect_bounds(&mut self, bounds: TimeRange)
Narrow the bounds of self to the overlap with bounds
Equivalent to .as_mut_range().intersect_bounds().
Sourcepub fn as_mut_range(&mut self) -> &mut TimeRange
pub fn as_mut_range(&mut self) -> &mut TimeRange
Mutably access the being-built time range.
This range is the intersection of all the ranges
from calls to incorporate_unwrap and
intersect_bounds.
CORRECTNESS
Normally it is only correct to narrow the range, not widen it. Getting the time range right is the responsibility of the caller.
Consider intersect_bounds instead.