TimerangeBound

Type Alias TimerangeBound 

pub type TimerangeBound<T> = TimeRangeBound<T>;
👎Deprecated:

use the new name, TimeRangeBound, instead

Deprecated compatibility alias for TimeRangeBound

Aliased Type

pub struct TimerangeBound<T> {
    obj: T,
    start: Option<SystemTime>,
    end: Option<SystemTime>,
}

Fields

obj: T

The underlying object, which we only want to expose if it is currently timely.

start: Option<SystemTime>

If present, when the object first became valid.

end: Option<SystemTime>

If present, when the object will no longer be valid.