Struct tor_checkable::signed::SignatureGated
source · pub struct SignatureGated<T> { /* private fields */ }Expand description
A SignatureGated object is a self-signed object that’s well-signed when one or more ValidatableSignature objects are correct.
Implementations
sourceimpl<T> SignatureGated<T>
impl<T> SignatureGated<T>
sourcepub fn new(obj: T, signatures: Vec<Box<dyn ValidatableSignature>>) -> Self
pub fn new(obj: T, signatures: Vec<Box<dyn ValidatableSignature>>) -> Self
Return a new SignatureGated object that will be treated as correct if every one if the given set of signatures is valid.